

set -ex



pip check
jupyter lab --version
jlpm --version
jlpm versions
jupyter labextension list
jupyter lab licenses
jupyter lab path
jupyter server extension list
jupyter serverextension list
jupyter server extension list 1>server_extensions 2>&1
cat server_extensions | rg "jupyterlab.*3\.6\.3.*OK"
jupyter serverextension list 1>serverextensions 2>&1
cat serverextensions | rg "jupyterlab.*3\.6\.3.*OK"
jupyter lab clean
exit 0
